"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const utils_1 = require("./utils");
function collectTypesFromArray(array, arrayName, file, createdInterfaces = []) {
    if (array.length === 0) {
        return {
            types: ['any'],
            interfaces: createdInterfaces
        };
    }
    const types = (array
        .map(value => {
        if (Array.isArray(value)) {
            const { types, interfaces } = collectTypesFromArray(value, `${arrayName}A`, file, createdInterfaces);
            createdInterfaces = interfaces;
            return types;
        }
        else if (value === null) {
            return 'null';
        }
        else if (typeof value === 'object') {
            const capitalizedKey = createdInterfaces.length === 0
                ? utils_1.capitalize(arrayName)
                : utils_1.capitalize(`${arrayName}${createdInterfaces.length}`);
            const { interface: createdInterface, interfaces: newInterfaces } = createInterface(value, capitalizedKey, file, '', createdInterfaces);
            const { interfaces, name } = utils_1.addOrRemoveInterface(createdInterface, newInterfaces);
            createdInterfaces = interfaces;
            return name;
        }
        return typeof value;
    })
        .reduce((a, b) => a.concat(b), [])
        .reduce((uniqTypes, type) => {
        if (!uniqTypes.includes(type)) {
            return [...uniqTypes, type];
        }
        return uniqTypes;
    }, []));
    return {
        types,
        interfaces: createdInterfaces
    };
}
function createInterface(object, objectName, file, parrentName = '', createdInterfaces = []) {
    const interfaceDeclaration = file.addInterface({
        name: objectName,
    });
    Object.keys(object).forEach(key => {
        const name = key;
        let typeName = typeof object[key];
        if (Array.isArray(object[key])) {
            const { types, interfaces } = collectTypesFromArray(object[key], key, file, createdInterfaces);
            typeName = utils_1.buildTypeStringOfArrayTypes(types);
            createdInterfaces = interfaces;
        }
        else if (object[key] === null) {
            typeName = 'null';
        }
        else if (typeof object[key] === 'object') {
            const capitalizeKey = utils_1.capitalize(key);
            typeName = `${parrentName ? parrentName : ''}${capitalizeKey}`;
            const { interface: createdInterface, interfaces: newInterfaces } = createInterface(object[key], typeName, file, capitalizeKey, createdInterfaces);
            const { interfaces, name } = utils_1.addOrRemoveInterface(createdInterface, newInterfaces);
            typeName = name;
            createdInterfaces = interfaces;
        }
        interfaceDeclaration.addProperty({
            name,
            type: typeName,
        });
    });
    return {
        interface: interfaceDeclaration,
        interfaces: createdInterfaces
    };
}
function createInterfacesFromObject(objectName, object) {
    const fileName = 'file.ts';
    const project = new ts_simple_ast_1.Project({
        useVirtualFileSystem: true,
        manipulationSettings: { indentationText: ts_simple_ast_1.IndentationText.TwoSpaces },
    });
    const fs = project.getFileSystem();
    const file = project.createSourceFile(fileName, '');
    createInterface(object, objectName, file);
    file.saveSync();
    return fs.readFileSync(fileName);
}
exports.createInterfacesFromObject = createInterfacesFromObject;
